# Workload Doc Markers

Workload document adds functionality to workload markers (see [workload markers](./workload-markers.md)
for more information) by allowing the user to inject documentation into the CRD that gets
generated by using a `+workload-docs` marker.  By injecting documentation to
the CRD, the consumer of the custom resource gets the added benefit by being
able to run `kubectl explain` against their resource and having documentation
right at their fingertips without having to navigate to API documentation in
order to see the usage of the API.

Documentation is injected by simply placing a multi-line `+workload-docs` marker
prior to the associated `+workload` marker.

Considering the following simple resource which is being managed by operator-builder:

```yaml
---
apiVersion: tenancy.platform.cnr.vmware.com/v1alpha1
kind: TanzuNamespace
metadata:
  name: cnpo-security-system #+workload:namespace:default="cnpo-security-system":type=string
spec:
  #+workload-docs: Defines the namespace in which all resources for this component will be placed into.
  tanzuNamespaceName: cnpo-security-system #+workload:namespace:default="cnpo-security-system":type=string
```

In this case, operator-builder will simply manage the `tanzuNamespaceName` field
as a workload marker which we are calling `namespace` in our custom CRD.  In addition
to managing that field, the prior `+workload-docs` marker is used as the docmentation
for that marker.  The previous `+workload-docs` field is always assigned to the very next
marker.  We wanted a separate marker definition for documentation to avoid very long lines
in which the documentation is a bit verbose.

The end state of this configuration is templated Go code which looks like so:

```go
// SecurityCommonComponentSpec defines the desired state of SecurityCommonComponent.
type SecurityCommonComponentSpec struct {
	// INSERT ADDITIONAL SPEC FIELDS - desired state of cluster
	// Important: Run "make" to regenerate code after modifying this file

	// +kubebuilder:default="cnpo-security-system"
	// +kubebuilder:validation:Optional
	// Defines the namespace in which all resources for this component will be placed into.
	Namespace string `json:"namespace"`
```

And when the `make manifests` command is run, the CRD is generated with
the documentation in the `description` filed which looks like:

```yaml
...
              namespace:
                default: cnpo-security-system
                description: Defines the namespace in which all resources for this
                  component will be placed into.
                type: string
...
```

Finally, the `kubectl explain` command now contains the documentation so that users
may see the documentation by way of `kubectl` and not any other tooling:

```bash
kubectl explain securitycommoncomponents.spec.namespace
KIND:     SecurityCommonComponent
VERSION:  security.cnpo.tanzulabs.vmware.com/v1alpha1

FIELD:    namespace <string>

DESCRIPTION:
     Defines the namespace in which all resources for this component will be
     placed into.
```

## API Documentation

API documentation in markdown can also be generated after building an operator.  To do
so, simply run the command `make docs` from the root of your repository.  This will build
the documentation in the `config/docs/apis.md` file.
